"""\
LurkenPlone skin installer.

Based on Martin Aspeli's MySkin product.
"""

from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.LurkenPlone import skin_globals

# Name of skin to set up
from Products.LurkenPlone.config import skinName

# Name of skin to copy as a starting point
from Products.LurkenPlone.config import baseSkin

# Name(s) of directory view(s) to insert into the skin path
from Products.LurkenPlone.config import skinFolders

# Name(s) of stylesheet(s) to register with portal_css
from Products.LurkenPlone.config import styleSheets

# Properties of stylsheet(s)
from Products.LurkenPlone.config import styleSheetProperties


def install(self, reinstall=False):
    """Installs the product.
    """
    out = []

    if reinstall:
        removeSkin(self, out)
        out.append('Uninstallation completed, reinstalling...')

    setupSkin(self, out)
    setDefaultSkin(self, out, skinName)

    out.append('Installation completed.')
    return '\n'.join(out)


def uninstall(self):
    """Uninstalls the product.
    """
    # Quickinstaller should uninstall the skin
    # automatically but it does not seem to...
    skinsTool = getToolByName(self, 'portal_skins')
    out = []

    removeSkin(self, out)
    if skinsTool.getDefaultSkin() == skinName:
        setDefaultSkin(self, out, 'Plone Default')

    out.append('Uninstallation completed.')
    return '\n'.join(out)


def reinstall(self):
    """Reinstalls the product.
    """
    return install(self, True)


def setupSkin(self, out):
    """Adds the directory views, registers the stylesheets, and
       sets up the new skin.
    """
    skinsTool = getToolByName(self, 'portal_skins')
    cssTool = getToolByName(self, 'portal_css')

    # Add directory view(s)

    addDirectoryViews(skinsTool, 'skins', skin_globals)
    if len(skinFolders) == 1:
        out.append("Added directory view %s to portal_skins." % skinFolders[0])
    elif len(skinFolders) > 1:
        out.append("Added directory views to portal_skins.")

    # Only add if the skin doesn't already exist
    if skinName not in skinsTool.getSkinSelections():

        # Get the skin paths of the base skin
        path = skinsTool.getSkinPath(baseSkin)
        path = [x.strip() for x in path.split(',')]

        # Add our own skin folders to the path, after 'custom'
        for skinFolder in skinFolders:
            if skinFolder not in path:
                try:
                    path.insert(path.index('custom')+1, skinFolder)
                except ValueError:
                    path.append(skinFolder)

        # Add our skin to the tool
        skinsTool.addSkinSelection(skinName, ','.join(path))
        out.append('Added skin %s to portal_skins.' % skinName)

        # Register our stylesheets
        for styleSheet in styleSheets:
            if styleSheet not in cssTool.getResourceIds():
                p = styleSheetProperties[styleSheet]
                cssTool.registerStylesheet(id=styleSheet, enabled=p['enabled'],
                                           title=p['title'], expression=p['expression'],
                                           cookable=p['cookable'], media=p['media'],
                                           rel=p['rel'], rendering=p['rendering'])
                out.append('Registered stylesheet %s with portal_css.' % styleSheet)

    else:
        out.append('Skin %s already exists, leaving it alone.' % skinName)


def setDefaultSkin(self, out, skinName):
    """Sets the default skin to skinName.
    """
    skinsTool = getToolByName(self, 'portal_skins')

    # Set our skin as default skin
    skinsTool.manage_properties(default_skin=skinName,
                                request_varname=skinsTool.getRequestVarname(),
                                allow_any=skinsTool.getAllowAny(),
                                cookie_persistence=skinsTool.getCookiePersistence())
    out.append('Set default skin to %s.' % skinName)


def removeSkin(self, out):
    """Removes the directory views, unregisters the stylesheets,
       and deletes the skin.
    """
    skinsTool = getToolByName(self, 'portal_skins')
    cssTool = getToolByName(self, 'portal_css')

    # Delete directory views (just in case)
    for skinFolder in skinFolders:
        if skinFolder in skinsTool.objectIds():
            skinsTool.manage_delObjects([skinFolder])
            out.append('Removed directory view %s from portal_skins.' % skinFolder)

    # Delete the property which holds the skin
    if skinName in skinsTool.getSkinSelections():
        skinsTool.manage_skinLayers(del_skin=1, chosen=(skinName,))
        out.append('Removed skin %s from portal_skins.' % skinName)

    # Unregister stylesheets (just in case)
    for styleSheet in styleSheets:
        if styleSheet in cssTool.getResourceIds():
            cssTool.unregisterResource(styleSheet)
            out.append('Unregistered stylesheet %s from portal_css.' % styleSheet)

